#include "defs.h"
#include "timer.h"

void hip_memcopy(int rank, int i, int ngpus) {
    real *d = nullptr, *h = nullptr;
    size_t N = 1<<26;
    size_t niters = 100, nwarmup = 23;
    h = (real *)calloc(N, sizeof(real));
    HIP_CALL(hipMalloc((void**)&d, sizeof(real)*N));
    HIP_CALL(hipMemset(d, 0, sizeof(real)*N));

    if (rank % i)
	    return;
    print(rank, "Memcopy benchmark:");
    if (i == 1) {
	    print(rank, "Using each GPU:");
    } else {
	    print(rank, std::string("Using every ") + std::to_string(i) + " GPU (" + std::to_string(ngpus/i) + " GPUs per node)");
    }

    {
	    timer t(rank2str(rank) + "H2D", sizeof(real)*N, niters);
	    for (int i = 0; i < niters + nwarmup; i++) {
		if (i == nwarmup)
			t.start();
		HIP_CALL(hipMemcpy(h, d, sizeof(real)*N, hipMemcpyHostToDevice));
	    }
    }
    {
	    timer t(rank2str(rank) + "D2H", sizeof(real)*N, niters);
	    for (int i = 0; i < niters + nwarmup; i++) {
		if (i == nwarmup)
			t.start();
		HIP_CALL(hipMemcpy(d, h, sizeof(real)*N, hipMemcpyDeviceToHost));
	    }
    }
    HIP_CALL(hipFree(d));
    free(h);
}

